/*
 * Copyright (c) 2008-2010 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 *    
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *    
 *  * Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of Nokia Corporation nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *    
 *    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *    DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 *    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *    
 *    Description:  
 */

#ifndef INTERNETEMAILCONTAINER_H
#define INTERNETEMAILCONTAINER_H

// INCLUDES
#include <coecntrl.h>

#include <aknlists.h> //for listbox
#include <eiklbo.h>   //for listboxobserver
#include <badesca.h>  //string arrays

#include "InternetEmailEngine.h" //for lazyness
#include "InternetEmailAppUi.h" //for lazyness
   
// FORWARD DECLARATIONS
class  CInternetEmailEngine;
class  CInternetEmailAppUi;

// CLASS DECLARATION

/**
*  CInternetEmailContainer  container control class.
*  
*/
class CInternetEmailContainer : public CCoeControl, MCoeControlObserver, MEikListBoxObserver, MDesCArray
    {
    public: // Constructors and destructor
        
        /**
        * EPOC default constructor.
        * @param aRect Frame rectangle for container.
        */
        void ConstructL(const TRect& aRect, CInternetEmailAppUi* aParent);

        /**
        * Destructor.
        */
        ~CInternetEmailContainer();

    public: // New functions
        /**
         * void MailCountChange()
         *  funtion called from engine observer 
         *  callback handlers when model changes are 
         *  occurring.
         */
        void MailCountChange();

    private: // New functions

        /**
         * void OpenEmailL()
         *  Calls generic messaging api and uses it to open
         *  selected message entry.
         */
        void OpenEmailL();
        
    public: // Functions from base classes

    private: // Functions from base classes
        // from MDesCArray
        TInt MdcaCount() const;
        TPtrC MdcaPoint(TInt aIndex) const;
        void EntryToListbox(TInt aIndex) const;

        /**
         * From CCoeControl, Handles key event.
         * @param aKeyEvent The key event.
         * @param aType The type of the event.
         * @return Indicates whether or not the key event was
         *           used by this control.
         */
         TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent,
                                    TEventCode aType);

        // From CoeControl,SizeChanged.
        void SizeChanged();
        TInt CountComponentControls() const;
        CCoeControl* ComponentControl(TInt aIndex) const;
        void Draw(const TRect& aRect) const;

        // MCoeControlObserver
        void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);

        // from MEikListBoxObserver
        void HandleListBoxEventL(CEikListBox* aListBox,TListBoxEvent aEventType);
        
        void HandleResourceChange(TInt aType);

    private: //data
        
        CEikTextListBox* iListBox;    // to view entries
        CInternetEmailAppUi* iParent; // for easier access to appui
        mutable TBuf<255> iText;      // because we are also an array

    };

#endif

// End of File
